@@ -5,7 +5,7 @@
 #include <sys/stat.h>
 #include <sys/time.h>
 
-static char *id = "@(#) $Id: cpuid.cc 1187 2005-10-13 21:33:55Z ezix $";
+static const char rcsid[] = "@(#) $Id: cpuid.cc 1319 2006-05-09 07:48:12Z ezix $";
 
 #if defined(__i386__) || defined(__alpha__)
 
@@ -250,6 +250,8 @@ static bool dointel(unsigned long maxi,
   if (!cpu)
     return false;
 
+  cpu->addHint("logo", string("intel"));
+
   if (maxi >= 1)
   {
     cpuid(cpunumber, 1, eax, ebx, ecx, edx);
@@ -411,6 +413,8 @@ static bool doamd(unsigned long maxi,
   if (maxi < 1)
     return false;
 
+  cpu->addHint("logo", string("amd"));
+
   cpuid(cpunumber, 1, eax, ebx, ecx, edx);
   stepping = eax & 0xf;
   model = (eax >> 4) & 0xf;
@@ -627,8 +631,6 @@ bool scan_cpuid(hwNode & n)
     currentcpu++;
   }
 
-  (void) &id;			// avoid warning "id defined but not used"
-
   return true;
 }
 
@@ -723,8 +725,6 @@ bool scan_cpuid(hwNode & n)
     currentcpu++;
   }
 
-  (void) &id;			// avoid warning "id defined but not used"
-
   return true;
 }
 
@@ -732,7 +732,6 @@ bool scan_cpuid(hwNode & n)
 
 bool scan_cpuid(hwNode & n)
 {
-  (void) &id;			// avoid warning "id defined but not used"
   return true;
 }
 #endif /* __alpha__ */
