@@ -18,8 +18,8 @@
 #include <unistd.h>
 #include <dirent.h>
 
-static char *id =
-  "@(#) $Id: device-tree.cc 1187 2005-10-13 21:33:55Z ezix $";
+static const char rcsid[] =
+  "@(#) $Id: device-tree.cc 1319 2006-05-09 07:48:12Z ezix $";
 
 #define DIMMINFOSIZE 0x80
 typedef __uint8_t dimminfo_buf[DIMMINFOSIZE];
@@ -352,10 +352,9 @@ static void scan_devtree_memory(hwNode & core)
 	  bank.setSlot(slotnames[i]);
 	  //bank.setPhysId(base);
 	  if (i < dimmtypes.size())
-	    bank.setProduct(dimmtypes[i]);
+	    bank.setDescription(dimmtypes[i]);
 	  if (i < dimmspeeds.size())
-	    bank.
-	      setProduct(hw::strip(bank.getProduct() + " " + dimmspeeds[i]));
+	    bank.setProduct(hw::strip(dimmspeeds[i]));
 	  bank.setSize(size);
 	  memory->addChild(bank);
 	}
@@ -503,7 +502,5 @@ bool scan_device_tree(hwNode & n)
     core->addCapability(get_string(DEVICETREE "/compatible"));
   }
 
-  (void) &id;			// avoid warning "id declared but not used"
-
   return true;
 }
