@@ -11,7 +11,7 @@
 
 using namespace hw;
 
-static char *id = "@(#) $Id: hw.cc 1187 2005-10-13 21:33:55Z ezix $";
+static const char rcsid[] = "@(#) $Id: hw.cc 1319 2006-05-09 07:48:12Z ezix $";
 
 struct hwNode_i
 {
@@ -95,8 +95,6 @@ hwNode::hwNode(const string & id,
   This->businfo = string("");
   This->physid = string("");
   This->dev = string("");
-
-  (void) &::id;			// avoid warning "id defined but not used"
 }
 
 hwNode::hwNode(const hwNode & o)
@@ -834,6 +832,15 @@ void hwNode::setConfig(const string & key,
     This->config.erase(This->config.find(key));
 }
 
+void hwNode::setConfig(const string & key,
+		       unsigned long long value)
+{
+  if (!This)
+    return;
+
+  This->config[key] = tostring(value);
+}
+
 string hwNode::getConfig(const string & key) const
 {
   if (!This)
