@@ -33,8 +33,7 @@
 #include <sys/types.h>
 using namespace std;
 
-static char *id =
-  "@(#) $Id: network.cc 1187 2005-10-13 21:33:55Z ezix $";
+static const char rcsid[] = "@(#) $Id: network.cc 1319 2006-05-09 07:48:12Z ezix $";
 
 #ifndef ARPHRD_IEEE1394
 #define ARPHRD_IEEE1394	24
@@ -436,7 +435,7 @@ bool scan_network(hwNode & n)
           interface.setCapacity(1000000000L);
         }
         if(ecmd.supported & SUPPORTED_Autoneg)
-          interface.addCapability("autonegociation", "Auto-negociation");
+          interface.addCapability("autonegotiation", "Auto-negotiation");
         
         switch(ecmd.speed)
         {
@@ -480,7 +479,7 @@ bool scan_network(hwNode & n)
             interface.setConfig("port", "fibre");
             break;
         }
-        interface.setConfig("autonegociation", (ecmd.autoneg == AUTONEG_DISABLE) ?  "off" : "on");
+        interface.setConfig("autonegotiation", (ecmd.autoneg == AUTONEG_DISABLE) ?  "off" : "on");
       }
 
       drvinfo.cmd = ETHTOOL_GDRVINFO;
@@ -524,6 +523,4 @@ bool scan_network(hwNode & n)
   }
   else
     return false;
-
-  (void) &id;			// avoid warning "id defined but not used"
 }
