@@ -14,8 +14,7 @@
 #include <linux/kdev_t.h>
 #endif
 
-static char *id =
-  "@(#) $Id: osutils.cc 1187 2005-10-13 21:33:55Z ezix $";
+static const char rcsid[] = "@(#) $Id: osutils.cc 1319 2006-05-09 07:48:12Z ezix $";
 
 using namespace std;
 
@@ -252,7 +251,6 @@ static string find_deventry(string basepath,
 string find_deventry(mode_t mode,
 		     dev_t device)
 {
-  (void) &id;			// avoid warning "id defined but not used"
   return find_deventry("/dev", mode, device);
 }
 
@@ -306,6 +304,15 @@ string lowercase(const string & s)
   return result;
 }
 
+string tostring(unsigned long long n)
+{
+  char buffer[80];
+
+  snprintf(buffer, sizeof(buffer), "%lld", n);
+
+  return string(buffer);
+}
+
 string join(const string & j, const string & s1, const string & s2)
 {
   if(s1 == "") return s2;
@@ -473,3 +480,32 @@ int open_dev(dev_t dev, const string & name)
   return -1;
 }				/* open_dev */
 
+#define putchar(c) ((char)(c & 0xff))
+
+string utf8(wchar_t c)
+{
+  string result = "";
+
+  if (c < 0x80) {
+    result += putchar (c);
+  }
+  else if (c < 0x800) {
+    result += putchar (0xC0 | c>>6);
+    result += putchar (0x80 | c & 0x3F);
+  }
+  else if (c < 0x10000) {
+    result += putchar (0xE0 | c>>12);
+    result += putchar (0x80 | c>>6 & 0x3F);
+    result += putchar (0x80 | c & 0x3F);
+  }
+  else if (c < 0x200000) {
+    result += putchar (0xF0 | c>>18);
+    result += putchar (0x80 | c>>12 & 0x3F);
+    result += putchar (0x80 | c>>6 & 0x3F);
+    result += putchar (0x80 | c & 0x3F);
+  }
+
+  return result;
+}
+
+
