@@ -19,8 +19,7 @@
 #include <unistd.h>
 #include <dirent.h>
 
-static char *id =
-  "@(#) $Id: parisc.cc 1187 2005-10-13 21:33:55Z ezix $";
+static const char rcsid[] = "@(#) $Id: parisc.cc 1319 2006-05-09 07:48:12Z ezix $";
 
 #define DEVICESPARISC "/sys/devices/parisc"
 
@@ -429,7 +428,5 @@ bool scan_parisc(hwNode & node)
   scan_device(*core);
   popd();
 
-  (void) &id;			// avoid warning "id declared but not used"
-
   return true;
 }
