@@ -6,7 +6,7 @@
 #include <unistd.h>
 #include <stdio.h>
 
-static char *id = "@(#) $Id: pci.cc 1187 2005-10-13 21:33:55Z ezix $";
+static const char rcsid[] = "@(#) $Id: pci.cc 1319 2006-05-09 07:48:12Z ezix $";
 
 #define PROC_BUS_PCI "/proc/bus/pci"
 #define PCIID_PATH "/usr/share/lshw/pci.ids:/usr/local/share/pci.ids:/usr/share/pci.ids:/etc/pci.ids:/usr/share/hwdata/pci.ids:/usr/share/misc/pci.ids"
@@ -23,6 +23,9 @@ static char *id = "@(#) $Id: pci.cc 1187 2005-10-13 21:33:55Z ezix $";
 #define PCI_PRIMARY_BUS         0x18	/* Primary bus number */
 #define PCI_SECONDARY_BUS       0x19	/* Secondary bus number */
 #define PCI_STATUS              0x06	/* 16 bits */
+#define PCI_LATENCY_TIMER	0x0d    /* 8 bits */
+#define PCI_SEC_LATENCY_TIMER	0x1b    /* Latency timer for secondary interface */
+#define PCI_CB_LATENCY_TIMER	0x1b    /* CardBus latency timer */
 #define PCI_STATUS_66MHZ       0x20	/* Support 66 Mhz PCI 2.1 bus */
 #define PCI_STATUS_CAP_LIST    0x10	/* Support Capability List */
 #define PCI_COMMAND_IO         0x1	/* Enable response in I/O space */
@@ -36,6 +39,9 @@ static char *id = "@(#) $Id: pci.cc 1187 2005-10-13 21:33:55Z ezix $";
 #define PCI_COMMAND_SERR       0x100	/* Enable SERR */
 #define PCI_COMMAND_FAST_BACK  0x200	/* Enable back-to-back writes */
 
+#define PCI_MIN_GNT		0x3e    /* 8 bits */
+#define PCI_MAX_LAT		0x3f    /* 8 bits */
+
 /*
  * The PCI interface treats multi-function devices as independent
  * devices.  The slot/function address of each device is encoded
@@ -691,6 +697,9 @@ bool scan_pci(hwNode & n)
       u_int16_t dclass = get_conf_word(d, PCI_CLASS_DEVICE);
       u_int16_t cmd = get_conf_word(d, PCI_COMMAND);
       u_int16_t status = get_conf_word(d, PCI_STATUS);
+      u_int8_t latency = get_conf_byte(d, PCI_LATENCY_TIMER);
+      u_int8_t min_gnt = get_conf_byte(d, PCI_MIN_GNT);
+      u_int8_t max_lat = get_conf_byte(d, PCI_MAX_LAT);
       u_int8_t progif = get_conf_byte(d, PCI_CLASS_PROG);
       u_int8_t rev = get_conf_byte(d, PCI_REVISION_ID);
       u_int8_t htype = get_conf_byte(d, PCI_HEADER_TYPE) & 0x7f;
@@ -709,10 +718,12 @@ bool scan_pci(hwNode & n)
           break;
         case PCI_HEADER_TYPE_BRIDGE:
           subsys_v = subsys_d = 0;
+          latency = get_conf_byte(d, PCI_SEC_LATENCY_TIMER);
           break;
         case PCI_HEADER_TYPE_CARDBUS:
           subsys_v = get_conf_word(d, PCI_CB_SUBSYSTEM_VENDOR_ID);
           subsys_d = get_conf_word(d, PCI_CB_SUBSYSTEM_ID);
+          latency = get_conf_byte(d, PCI_CB_LATENCY_TIMER);
           break;
       }
 
@@ -729,6 +740,8 @@ bool scan_pci(hwNode & n)
         addHints(host, d.vendor_id, d.device_id, subsys_v, subsys_d, dclass);
 	host.claim();
 	host.setBusInfo(businfo);
+        if(latency)
+          host.setConfig("latency", latency);
 	if (d.size[0] > 0)
 	  host.setPhysId(d.base_addr[0] & PCI_ADDR_MEM_MASK);
 
@@ -827,6 +840,11 @@ bool scan_pci(hwNode & n)
 	  if (deviceclass == hw::bridge || deviceclass == hw::storage)
 	    device->addCapability(devicename);
 
+          if(device->isCapable("isa") ||
+             device->isCapable("pci") ||
+             device->isCapable("agp"))
+            device->claim();
+
 	  scan_resources(*device, d);
 
 	  if (deviceclass == hw::display)
@@ -837,8 +855,7 @@ bool scan_pci(hwNode & n)
 
 	  if (dclass == PCI_CLASS_BRIDGE_PCI)
 	  {
-	    device->
-	      setHandle(pci_bushandle(get_conf_byte(d, PCI_SECONDARY_BUS)));
+	    device->setHandle(pci_bushandle(get_conf_byte(d, PCI_SECONDARY_BUS)));
 	    device->claim();
 	  }
 	  else
@@ -847,6 +864,11 @@ bool scan_pci(hwNode & n)
 
 	    snprintf(irq, sizeof(irq), "%d", d.irq);
 	    device->setHandle(pci_handle(d.bus, d.dev, d.func));
+            device->setConfig("latency", latency);
+            if(max_lat)
+              device->setConfig("maxlatency", max_lat);
+            if(min_gnt)
+              device->setConfig("mingnt", min_gnt);
 	    if (d.irq != 0)
 	    {
 	      //device->setConfig("irq", irq);
@@ -910,7 +932,5 @@ bool scan_pci(hwNode & n)
     fclose(f);
   }
 
-  (void) &id;			// avoid warning "id defined but not used"
-
   return false;
 }
