@@ -12,11 +12,46 @@
 #include "version.h"
 #include "osutils.h"
 #include <iostream>
+#include <sstream>
 #include <iomanip>
 #include <unistd.h>
 #include <stdio.h>
+#include <sys/ioctl.h>
+#include <termios.h>
 
-static char *id = "@(#) $Id: print.cc 1187 2005-10-13 21:33:55Z ezix $";
+static const char rcsid[] = "@(#) $Id: print.cc 1319 2006-05-09 07:48:12Z ezix $";
+
+static unsigned int columns = 0, rows = 0;
+
+static void init_wsize()
+{
+  struct winsize ws;
+  char *env;
+ 
+  if (!ioctl(STDOUT_FILENO, TIOCGWINSZ, &ws))
+  {
+    rows = ws.ws_row;
+    columns = ws.ws_col;
+  }
+ 
+  if (!rows)
+  {
+    env = getenv("LINES");
+    if (env)
+      rows = atoi(env);
+    if (!rows)
+      rows = 24;
+  }
+
+  if (!columns)
+  {
+    env = getenv("COLUMNS");
+    if (env)
+    columns = atoi(env);
+    if (!columns)
+    columns = 80;
+  }
+}
 
 static void tab(int level,
 		bool connect = true)
@@ -32,7 +67,7 @@ static void tab(int level,
     cout << "  ";
 }
 
-static void decimalkilos(unsigned long long value)
+static void decimalkilos(ostream & out, unsigned long long value)
 {
   const char *prefixes = "KMGTPEZY";
   unsigned int i = 0;
@@ -43,12 +78,12 @@ static void decimalkilos(unsigned long long value)
     i++;
   }
 
-  cout << value;
+  out << value;
   if ((i > 0) && (i <= strlen(prefixes)))
-    cout << prefixes[i - 1];
+    out << prefixes[i - 1];
 }
 
-static void kilobytes(unsigned long long value)
+static void kilobytes(ostream & out, unsigned long long value)
 {
   const char *prefixes = "KMGTPEZY";
   unsigned int i = 0;
@@ -59,10 +94,10 @@ static void kilobytes(unsigned long long value)
     i++;
   }
 
-  cout << value;
+  out << value;
   if ((i > 0) && (i <= strlen(prefixes)))
-    cout << prefixes[i - 1];
-  cout << "B";
+    out << prefixes[i - 1];
+  out << "B";
 }
 
 void print(hwNode & node,
@@ -89,6 +124,16 @@ void print(hwNode & node,
     cout << "<head>" << endl;
     cout << "<meta name=\"generator\" " <<
       " content=\"lshw-" << getpackageversion() << "\">" << endl;
+    cout << "<style type=\"text/css\">" << endl;
+    cout << "  .first {font-weight: bold; margin-left: none; padding-right: 1em;vertical-align: top; }" << endl;
+    cout << "  .second {padding-left: 1em; width: 100%; vertical-align: center; }" << endl;
+    cout << "  .id {font-family: monospace;}" << endl;
+    cout << "  .indented {margin-left: 2em; border-left: dotted thin #dde; padding-bottom: 1em; }" << endl;
+    cout << "  .node {border: solid thin #ffcc66; padding: 1em; background: #ffffcc; }" << endl;
+    cout << "  .node-unclaimed {border: dotted thin #c3c3c3; padding: 1em; background: #fafafa; }" << endl;
+    cout << "  .node-disabled {border: solid thin #f55; padding: 1em; background: #fee; }" << endl;
+    cout << "</style>" << endl;
+
     cout << "<title>";
     cout << node.getId();
     cout << "</title>" << endl;
@@ -101,37 +146,50 @@ void print(hwNode & node,
     tab(level, !html);
 
     if (html)
-      cout << "<b>";
+    {
+      tab(level, false);
+      cout << "<div class=\"indented\">" << endl;
+      tab(level, false);
+      cout << "<table width=\"100%\" class=\"node";
+      if(!node.claimed())
+        cout << "-unclaimed";
+      else
+      {
+        if(node.disabled())
+          cout << "-disabled";
+      }
+      cout << "\" summary=\"attributes of " << node.getId() << "\">" << endl;
+    }
+
+    if (html)
+      cout << " <thead><tr><td class=\"first\">id:</td><td class=\"second\"><div class=\"id\">";
     if (html && (!node.claimed() || node.disabled()))
       cout << "<font color=\"gray\">";
     cout << node.getId();
-    if (node.disabled())
-      cout << " DISABLED";
-    if (!node.claimed())
-      cout << " UNCLAIMED";
     if (html && (!node.claimed() || node.disabled()))
       cout << "</font>";
     if (html)
-      cout << "</b><br>";
+      cout << "</div>";
+    else
+    {
+      if (node.disabled())
+        cout << " DISABLED";
+      if (!node.claimed())
+        cout << " UNCLAIMED";
+    }
+    if (html)
+      cout << "</td></tr></thead>";
     cout << endl;
 
-    if (html)
-    {
-      tab(level, false);
-      cout << "<div style=\"margin-left: 2em\">" << endl;
-      tab(level, false);
-      cout << "<table bgcolor=\"#e8e0e0\"";
-      cout << " summary=\"attributes of " << node.getId() << "\">" << endl;
-    }
 #if 0
     if (node.getHandle() != "")
     {
       tab(level + 1, false);
       if (html)
-	cout << "<tr><td>";
+	cout << "<tr><td class=\"first\">";
       cout << "handle: ";
       if (html)
-	cout << "</td><td>";
+	cout << "</td><td class=\"second\">";
       cout << node.getHandle();
       if (html)
 	cout << "</td></tr>";
@@ -139,14 +197,17 @@ void print(hwNode & node,
     }
 #endif
 
+    if (html)
+      cout << " <tbody>" << endl;
+
     if (node.getDescription() != "")
     {
       tab(level + 1, false);
       if (html)
-	cout << "<tr><td>";
+	cout << "<tr><td class=\"first\">";
       cout << "description: ";
       if (html)
-	cout << "</td><td>";
+	cout << "</td><td class=\"second\">";
       cout << node.getDescription();
       if (html)
 	cout << "</td></tr>";
@@ -157,10 +218,10 @@ void print(hwNode & node,
     {
       tab(level + 1, false);
       if (html)
-	cout << "<tr><td>";
+	cout << "<tr><td class=\"first\">";
       cout << "product: ";
       if (html)
-	cout << "</td><td>";
+	cout << "</td><td class=\"second\">";
       cout << node.getProduct();
       if (html)
 	cout << "</td></tr>";
@@ -171,10 +232,10 @@ void print(hwNode & node,
     {
       tab(level + 1, false);
       if (html)
-	cout << "<tr><td>";
+	cout << "<tr><td class=\"first\">";
       cout << "vendor: ";
       if (html)
-	cout << "</td><td>";
+	cout << "</td><td class=\"second\">";
       cout << node.getVendor();
       if (html)
 	cout << "</td></tr>";
@@ -185,13 +246,13 @@ void print(hwNode & node,
     {
       tab(level + 1, false);
       if (html)
-	cout << "<tr><td>";
+	cout << "<tr><td class=\"first\">";
       cout << "physical id: ";
       if (html)
-	cout << "</td><td>";
+	cout << "</td><td class=\"second\"><div class=\"id\">";
       cout << node.getPhysId();
       if (html)
-	cout << "</td></tr>";
+	cout << "</div></td></tr>";
       cout << endl;
     }
 
@@ -199,13 +260,13 @@ void print(hwNode & node,
     {
       tab(level + 1, false);
       if (html)
-	cout << "<tr><td>";
+	cout << "<tr><td class=\"first\">";
       cout << "bus info: ";
       if (html)
-	cout << "</td><td>";
+	cout << "</td><td class=\"second\"><div class=\"id\">";
       cout << node.getBusInfo();
       if (html)
-	cout << "</td></tr>";
+	cout << "</div></td></tr>";
       cout << endl;
     }
 
@@ -217,13 +278,13 @@ void print(hwNode & node,
       {
       	tab(level + 1, false);
       	if (html)
-		cout << "<tr><td>";
+		cout << "<tr><td class=\"first\">";
       	cout << "logical name: ";
       	if (html)
-		cout << "</td><td>";
+		cout << "</td><td class=\"second\"><div class=\"id\">";
       	cout << logicalnames[i];
       	if (html)
-		cout << "</td></tr>";
+		cout << "</div></td></tr>";
       	cout << endl;
       }
     }
@@ -232,10 +293,10 @@ void print(hwNode & node,
     {
       tab(level + 1, false);
       if (html)
-	cout << "<tr><td>";
+	cout << "<tr><td class=\"first\">";
       cout << "version: ";
       if (html)
-	cout << "</td><td>";
+	cout << "</td><td class=\"second\">";
       cout << node.getVersion();
       if (html)
 	cout << "</td></tr>";
@@ -246,10 +307,10 @@ void print(hwNode & node,
     {
       tab(level + 1, false);
       if (html)
-	cout << "<tr><td>";
+	cout << "<tr><td class=\"first\">";
       cout << "serial: ";
       if (html)
-	cout << "</td><td>";
+	cout << "</td><td class=\"second\">";
       cout << node.getSerial();
       if (html)
 	cout << "</td></tr>";
@@ -260,10 +321,10 @@ void print(hwNode & node,
     {
       tab(level + 1, false);
       if (html)
-	cout << "<tr><td>";
+	cout << "<tr><td class=\"first\">";
       cout << "slot: ";
       if (html)
-	cout << "</td><td>";
+	cout << "</td><td class=\"second\">";
       cout << node.getSlot();
       if (html)
 	cout << "</td></tr>";
@@ -274,10 +335,10 @@ void print(hwNode & node,
     {
       tab(level + 1, false);
       if (html)
-	cout << "<tr><td>";
+	cout << "<tr><td class=\"first\">";
       cout << "size: ";
       if (html)
-	cout << "</td><td>";
+	cout << "</td><td class=\"second\">";
       switch (node.getClass())
       {
       case hw::display:
@@ -285,7 +346,7 @@ void print(hwNode & node,
       case hw::address:
       case hw::storage:
       case hw::disk:
-	kilobytes(node.getSize());
+	kilobytes(cout, node.getSize());
 	if (html)
 	  cout << "</td></tr>";
 	break;
@@ -293,14 +354,14 @@ void print(hwNode & node,
       case hw::processor:
       case hw::bus:
       case hw::system:
-	decimalkilos(node.getSize());
+	decimalkilos(cout, node.getSize());
 	cout << "Hz";
 	if (html)
 	  cout << "</td></tr>";
 	break;
 
       case hw::network:
-	decimalkilos(node.getSize());
+	decimalkilos(cout, node.getSize());
 	cout << "B/s";
 	if (html)
 	  cout << "</td></tr>";
@@ -325,17 +386,17 @@ void print(hwNode & node,
     {
       tab(level + 1, false);
       if (html)
-	cout << "<tr><td>";
+	cout << "<tr><td class=\"first\">";
       cout << "capacity: ";
       if (html)
-	cout << "</td><td>";
+	cout << "</td><td class=\"second\">";
       switch (node.getClass())
       {
       case hw::memory:
       case hw::address:
       case hw::storage:
       case hw::disk:
-	kilobytes(node.getCapacity());
+	kilobytes(cout, node.getCapacity());
 	if (html)
 	  cout << "</td></tr>";
 	break;
@@ -343,14 +404,14 @@ void print(hwNode & node,
       case hw::processor:
       case hw::bus:
       case hw::system:
-	decimalkilos(node.getCapacity());
+	decimalkilos(cout, node.getCapacity());
 	cout << "Hz";
 	if (html)
 	  cout << "</td></tr>";
 	break;
 
       case hw::network:
-	decimalkilos(node.getCapacity());
+	decimalkilos(cout, node.getCapacity());
 	cout << "B/s";
 	if (html)
 	  cout << "</td></tr>";
@@ -375,7 +436,7 @@ void print(hwNode & node,
     {
       tab(level + 1, false);
       if (html)
-	cout << "<tr><td>";
+	cout << "<tr><td class=\"first\">";
       if (node.getSize() == 0)
 	cout << "address: " << hex << setfill('0') << setw(8) << node.
 	  getStart() << dec;
@@ -390,10 +451,10 @@ void print(hwNode & node,
     {
       tab(level + 1, false);
       if (html)
-	cout << "<tr><td>";
+	cout << "<tr><td class=\"first\">";
       cout << "width: ";
       if (html)
-	cout << "</td><td>";
+	cout << "</td><td class=\"second\">";
       cout << node.getWidth() << " bits";
       if (html)
 	cout << "</td></tr>";
@@ -404,14 +465,14 @@ void print(hwNode & node,
     {
       tab(level + 1, false);
       if (html)
-	cout << "<tr><td>";
+	cout << "<tr><td class=\"first\">";
       cout << "clock: ";
       if (html)
-	cout << "</td><td>";
-      decimalkilos(node.getClock());
+	cout << "</td><td class=\"second\">";
+      decimalkilos(cout, node.getClock());
       cout << "Hz";
       if (node.getClass() == hw::memory)
-	cout << " (" << 1.0e9 / node.getClock() << "ns)";
+	cout << " (" << setiosflags(ios::fixed) << setprecision(1) << 1.0e9 / node.getClock() << "ns)";
       if (html)
 	cout << "</td></tr>";
       cout << endl;
@@ -421,10 +482,10 @@ void print(hwNode & node,
     {
       tab(level + 1, false);
       if (html)
-	cout << "<tr><td>";
+	cout << "<tr><td class=\"first\">";
       cout << "capabilities: ";
       if (html)
-	cout << "</td><td>";
+	cout << "</td><td class=\"second\">";
       cout << node.getCapabilities();
       if (html)
 	cout << "</td></tr>";
@@ -435,15 +496,15 @@ void print(hwNode & node,
     {
       tab(level + 1, false);
       if (html)
-	cout << "<tr><td>";
+	cout << "<tr><td class=\"first\">";
       cout << "configuration:";
       if (html)
-	cout << "</td><td><table summary=\"configuration of " << node.
+	cout << "</td><td class=\"second\"><table summary=\"configuration of " << node.
 	  getId() << "\">";
       for (unsigned int i = 0; i < config.size(); i++)
       {
 	if (html)
-	  cout << "<tr><td>";
+	  cout << "<tr><td class=\"sub-first\">";
 	cout << " " << config[i];
 	if (html)
 	  cout << "</td></tr>";
@@ -457,15 +518,15 @@ void print(hwNode & node,
     {
       tab(level + 1, false);
       if (html)
-	cout << "<tr><td>";
+	cout << "<tr><td class=\"first\">";
       cout << "resources:";
       if (html)
-	cout << "</td><td><table summary=\"resources of " << node.
+	cout << "</td><td class=\"second\"><table summary=\"resources of " << node.
 	  getId() << "\">";
       for (unsigned int i = 0; i < resources.size(); i++)
       {
 	if (html)
-	  cout << "<tr><td>";
+	  cout << "<tr><td class=\"sub-first\">";
 	cout << " " << resources[i];
 	if (html)
 	  cout << "</td></tr>";
@@ -475,17 +536,24 @@ void print(hwNode & node,
       cout << endl;
     }
 
+    if (html)
+      cout << " </tbody>";
+
     if (html)
     {
       tab(level, false);
-      cout << "</table>" << endl;
+      cout << "</table></div>" << endl;
     }
 
   }				// if visible
 
   for (unsigned int i = 0; i < node.countChildren(); i++)
   {
+    if(html)
+      cout << "<div class=\"indented\">" << endl;
     print(*node.getChild(i), html, visible(node.getClassName()) ? level + 1 : 1);
+    if(html)
+      cout << "</div>" << endl;
   }
 
   if (html)
@@ -493,7 +561,6 @@ void print(hwNode & node,
     if (visible(node.getClassName()))
     {
       tab(level, false);
-      cout << "</div>" << endl;
     }
     if (level == 0)
     {
@@ -501,8 +568,6 @@ void print(hwNode & node,
       cout << "</html>" << endl;
     }
   }
-
-  (void) &id;			// avoid "id defined but not used" warning
 }
 
 struct hwpath
@@ -522,9 +587,16 @@ static void printhwnode(hwNode & node,
   entry.path = "";
   if (node.getPhysId() != "")
     entry.path = prefix + "/" + node.getPhysId();
-  entry.description = node.getProduct();
+  if(node.getClass() != hw::memory)
+    entry.description = node.getProduct(); // memory devices tend to have obscure product names
   if (entry.description == "")
     entry.description = node.getDescription();
+  if(node.getSize() && ((node.getClass() == hw::memory) || (node.getClass() == hw::disk) || (node.getClass() == hw::storage)))
+  {
+    ostringstream size;
+    kilobytes(size, node.getSize());
+    entry.description = size.str() + " " + entry.description;
+  }
   entry.devname = node.getLogicalName();
   entry.classname = node.getClassName();
 
@@ -556,12 +628,23 @@ static void printbusinfo(hwNode & node,
   }
 }
 
+static void printline(ostringstream & out)
+{
+  string s = out.str();
+
+  if(isatty(STDOUT_FILENO) && (s.length()>columns))
+    s.erase(columns);
+  cout << s << endl;
+  out.str("");
+}
+
 static void printincolumns( vector < hwpath > &l, const char *cols[])
 {
   unsigned int l1 = strlen(cols[0]),
     l2 = strlen(cols[1]),
     l3 = strlen(cols[2]);
   unsigned int i = 0;
+  ostringstream out;
 
   for (i = 0; i < l.size(); i++)
   {
@@ -573,28 +656,29 @@ static void printincolumns( vector < hwpath > &l, const char *cols[])
       l3 = l[i].classname.length();
   }
 
-  cout << cols[0];
-  cout << spaces(2 + l1 - strlen(cols[0]));
-  cout << cols[1];
-  cout << spaces(2 + l2 - strlen(cols[1]));
-  cout << cols[2];
-  cout << spaces(2 + l3 - strlen(cols[2]));
-  cout << cols[3];
-  cout << endl;
+  out << cols[0];
+  out << spaces(2 + l1 - strlen(cols[0]));
+  out << cols[1];
+  out << spaces(2 + l2 - strlen(cols[1]));
+  out << cols[2];
+  out << spaces(2 + l3 - strlen(cols[2]));
+  out << cols[3];
+  printline(out);
 
-  cout << spaces(l1 + l2 + l3 + strlen(cols[3]) + 6, "=");
-  cout << endl;
+  out << spaces(l1 + l2 + l3 + strlen(cols[3]) + 6, "=");
+  printline(out);
 
   for (i = 0; i < l.size(); i++)
     if (visible(l[i].classname.c_str()))
     {
-      cout << l[i].path;
-      cout << spaces(2 + l1 - l[i].path.length());
-      cout << l[i].devname;
-      cout << spaces(2 + l2 - l[i].devname.length());
-      cout << l[i].classname;
-      cout << spaces(2 + l3 - l[i].classname.length());
-      cout << l[i].description << endl;
+      out << l[i].path;
+      out << spaces(2 + l1 - l[i].path.length());
+      out << l[i].devname;
+      out << spaces(2 + l2 - l[i].devname.length());
+      out << l[i].classname;
+      out << spaces(2 + l3 - l[i].classname.length());
+      out << l[i].description;
+      printline(out);
     }
 }
 
@@ -608,6 +692,7 @@ void printhwpath(hwNode & node)
 {
   vector < hwpath > l;
   printhwnode(node, l);
+  init_wsize();
   printincolumns(l, hwpathcols);
 }
 
@@ -621,5 +706,6 @@ void printbusinfo(hwNode & node)
 {
   vector < hwpath > l;
   printbusinfo(node, l);
+  init_wsize();
   printincolumns(l, businfocols);
 }
