@@ -20,7 +20,7 @@
 #include <string>
 #include <map>
 
-static char *id = "@(#) $Id: scsi.cc 1187 2005-10-13 21:33:55Z ezix $";
+static const char rcsid[] = "@(#) $Id: scsi.cc 1319 2006-05-09 07:48:12Z ezix $";
 
 #define SG_X "/dev/sg%d"
 #define SG_MAJOR 21
@@ -678,6 +678,7 @@ static bool scan_sg(int sg,
   switch (m_id.scsi_type)
   {
   case 0:
+  case 14:
     device = hwNode("disk", hw::disk);
     break;
   case 1:
@@ -715,7 +716,7 @@ static bool scan_sg(int sg,
   do_inquiry(fd, device);
   if ((m_id.scsi_type == 4) || (m_id.scsi_type == 5))
     scan_cdrom(device);
-  if ((m_id.scsi_type == 0) || (m_id.scsi_type == 7))
+  if ((m_id.scsi_type == 0) || (m_id.scsi_type == 7) || (m_id.scsi_type == 14))
     scan_disk(device);
 
   memset(slot_name, 0, sizeof(slot_name));
@@ -869,7 +870,5 @@ bool scan_scsi(hwNode & n)
 
   scan_hosts(n);
 
-  (void) &id;
-
   return false;
 }
