@@ -14,7 +14,7 @@
 #include <unistd.h>
 #include <stdio.h>
 
-static char *id = "@(#) $Id: print.cc 1187 2005-10-13 21:33:55Z ezix $";
+static char *id = "@(#) $Id: print.cc 1319 2006-05-09 07:48:12Z ezix $";
 
 static string itos(unsigned long long value)
 {
@@ -164,6 +164,9 @@ void printmarkup(const hwNode & node, GtkTextView *textview, const string & hwpa
   if(node.getHint("bus.icon").defined())
     inserticon(string("lshw-") + node.getHint("bus.icon").asString(), "", buffer, iter, textview);
 
+  if(node.getHint("logo").defined())
+    inserticon(string("lshw-") + node.getHint("logo").asString(), "", buffer, iter, textview);
+
   gtk_text_buffer_insert (buffer, &iter, "\n\n", -1);
 
   //out << printattr("description", node.getDescription());
